@@ -45,7 +45,7 @@ namespace Npgsql.SqlGenerators
             // almost the opposite of limit, need to skip first.
             VisitedExpression skip = expression.Input.Expression.Accept(this);
             InputExpression input;
-            if (!(skip is ProjectionExpression))
+            if (!(skip is ProjectionExpression) || !(((ProjectionExpression)skip).From is FromExpression))
             {
                 input = CheckedConvertFrom(skip, expression.Input.VariableName);
                 // return this value
@@ -54,13 +54,10 @@ namespace Npgsql.SqlGenerators
             else
             {
                 input = ((ProjectionExpression)skip).From;
-                if (input is FromExpression)
-                {
-                    if (_variableSubstitution.ContainsKey(((FromExpression)input).Name))
-                        _variableSubstitution[expression.Input.VariableName] = _variableSubstitution[((FromExpression)input).Name];
-                    else
-                        _variableSubstitution[expression.Input.VariableName] = ((FromExpression)input).Name;
-                }
+                if (_variableSubstitution.ContainsKey(((FromExpression)input).Name))
+                    _variableSubstitution[expression.Input.VariableName] = _variableSubstitution[((FromExpression)input).Name];
+                else
+                    _variableSubstitution[expression.Input.VariableName] = ((FromExpression)input).Name;
             }
             OrderByExpression orderBy = new OrderByExpression();
             foreach (var order in expression.SortOrder)
@@ -109,8 +106,23 @@ namespace Npgsql.SqlGenerators
             // may come from a join or a select
             //return new LiteralExpression(expression.Target.MetadataProperties.TryGetValue(
             // replace with better solution
+            MetadataProperty metadata;
+            string tableName;
+            string overrideTable = "http://schemas.microsoft.com/ado/2007/12/edm/EntityStoreSchemaGenerator:Name";
+            if (expression.Target.MetadataProperties.TryGetValue(overrideTable, false, out metadata) && metadata.Value != null)
+            {
+                tableName = metadata.Value.ToString();
+            }
+            else if (expression.Target.MetadataProperties.TryGetValue("Table", false, out metadata) && metadata.Value != null)
+            {
+                tableName = metadata.Value.ToString();
+            }
+            else
+            {
+                tableName = expression.Target.Name;
+            }
             if (_projectVarName.Count != 0) // this can happen in dml
-                _variableSubstitution[_projectVarName.Peek()] = expression.Target.Name;
+                _variableSubstitution[_projectVarName.Peek()] = tableName;
             SubstituteFilterVar(expression.Target.Name);
             if (expression.Target.MetadataProperties.Contains("DefiningQuery"))
             {
@@ -121,7 +133,6 @@ namespace Npgsql.SqlGenerators
                 }
             }
 
-            MetadataProperty metadata;
             LiteralExpression scan;
             string overrideSchema = "http://schemas.microsoft.com/ado/2007/12/edm/EntityStoreSchemaGenerator:Schema";
             if (expression.Target.MetadataProperties.TryGetValue(overrideSchema, false, out metadata) && metadata.Value != null)
@@ -137,15 +148,7 @@ namespace Npgsql.SqlGenerators
                 scan = new LiteralExpression(QuoteIdentifier(expression.Target.EntityContainer.Name));
             }
             scan.Append(".");
-            string overrideTable = "http://schemas.microsoft.com/ado/2007/12/edm/EntityStoreSchemaGenerator:Name";
-            if (expression.Target.MetadataProperties.TryGetValue(overrideTable, false, out metadata) && metadata.Value != null)
-            {
-                scan.Append(QuoteIdentifier(metadata.Value.ToString()));
-            }
-            else
-            {
-                scan.Append(QuoteIdentifier(expression.Target.Name));
-            }
+            scan.Append(QuoteIdentifier(tableName));
 
             return scan;
         }
@@ -775,6 +778,18 @@ namespace Npgsql.SqlGenerators
                     case "Left":
                         System.Diagnostics.Debug.Assert(args.Count == 2);
                         return Substring(args[0].Accept(this), new LiteralExpression(" 1 "), args[1].Accept(this));
+                    case "Right":
+                        System.Diagnostics.Debug.Assert(args.Count == 2);
+                        {
+                            var arg0 = args[0].Accept(this);
+                            var arg1 = args[1].Accept(this);
+                            var start = new FunctionExpression("char_length");
+                            start.AddArgument(arg0);
+                            // add one before subtracting count since strings are 1 based in postgresql
+                            start.Append("+1-");
+                            start.Append(arg1);
+                            return Substring(arg0, start);
+                        }
                     case "Substring":
                         System.Diagnostics.Debug.Assert(args.Count == 3);
                         return Substring(args[0].Accept(this), args[1].Accept(this), args[2].Accept(this));
@@ -864,8 +879,6 @@ namespace Npgsql.SqlGenerators
                     case "NewGuid":
                         return new FunctionExpression("uuid_generate_v4");
 
-                    case "Right":
-                    // TODO: need to clone expression for use in substring and length
                     default:
                         throw new NotSupportedException("NotSupported " + function.Name);
                 }
@@ -882,6 +895,14 @@ namespace Npgsql.SqlGenerators
             return substring;
         }
 
+        private VisitedExpression Substring(VisitedExpression source, VisitedExpression start)
+        {
+            FunctionExpression substring = new FunctionExpression("substr");
+            substring.AddArgument(source);
+            substring.AddArgument(start);
+            return substring;
+        }
+
         private VisitedExpression UnaryMath(string funcName, IList<DbExpression> args)
         {
             FunctionExpression mathFunction = new FunctionExpression(funcName);
